
--todo:
--make precast be negative timeline
--make it identify the combat start and add a special entry/block for it
--implement divisors in the timeline (vertical lines)
--the timeline need to identify this spelcial block and add a divisor
 

do
	local Details = Details
	if (not Details) then
		print ("Details! Not Found.")
		return
	end
	
	local _
	local DF = DetailsFramework

	--> minimal details version required to run this plugin
	local MINIMAL_DETAILS_VERSION_REQUIRED = 136
	local DETAILS_ATTRIBUTE_DAMAGE = DETAILS_ATTRIBUTE_DAMAGE
	local CASTHISTORY_VERSION = "v1.0.0"
	
	--> create a plugin object
	local castHistory = Details:NewPluginObject ("Details_CastHistory", DETAILSPLUGIN_ALWAYSENABLED)
	--> just localizing here the plugin's main frame
	local frame = castHistory.Frame
	--> set the description
	castHistory:SetPluginDescription ("Show a time line of casts of players")
	--> get the framework object
	local framework = castHistory:GetFramework()
	
	--> spells cache
	local spellCache = {}
	local auraCache = {}
	
	--> when receiving an event from details, handle it here
	local handle_details_event = function (event, ...)
	
		if (event == "COMBAT_PLAYER_ENTER") then
			castHistory.OnCombatStart (...)

		elseif (event == "COMBAT_PLAYER_LEAVE") then
			castHistory.OnCombatEnd()
			
		elseif (event == "PLUGIN_DISABLED") then
			--> plugin has been disabled at the details options panel
		
		elseif (event == "PLUGIN_ENABLED") then
			--> plugin has been enabled at the details options panel
		
		elseif (event == "DETAILS_DATA_SEGMENTREMOVED") then
			--> old segment got deleted by the segment limit
		
		elseif (event == "DETAILS_DATA_RESET") then
			--> combat data got wiped

		end
		
	end
	
	--runs on each member in the group
	function castHistory.BuildPlayerList (unitId, combatObject)
		local GUID = UnitGUID (unitId)
		combatObject.spells_cast_timeline [GUID] = {}
		combatObject.aura_timeline [GUID] = {}
	end
	
	--runs on each member in the group
	function castHistory.CacheTransfer (unitId, combatObject, spellCache, auraCache)
		local GUID = UnitGUID (unitId)
		local cachedCastTable = spellCache [GUID]
		local combatCastTable = combatObject.spells_cast_timeline [GUID]
		
		if (cachedCastTable and combatCastTable) then
			local cutOffTime = GetTime() - 5
			for i = #cachedCastTable, 1, -1 do
				if (cachedCastTable [i][1] < cutOffTime) then
					tremove (cachedCastTable, i)
				end
			end

			for i = 1, #cachedCastTable do
				tinsert (combatCastTable, cachedCastTable[i])
			end
			
			table.wipe (cachedCastTable)
		end	
		
		--
		
		local cachedAuraTable = auraCache [GUID]
		local combatAuraTable = combatObject.aura_timeline [GUID]
		
		if (cachedAuraTable and combatAuraTable) then
			for spellId, appliedAt in pairs (cachedAuraTable) do
			
				--check if the aura is still up
				local auraUp
				for i = 1, 40 do
					local name, texture, count, debuffType, duration, expirationTime, caster, canStealOrPurge, nameplateShowPersonal, auraSpellId = UnitBuff (unitId, i)
					
					if (not name) then
						return
					end
					
					if (spellId == auraSpellId) then
						auraUp = true
						break
					end
				end
				
				if (auraUp) then
					combatAuraTable [spellId] = {
						enabled = true,
						appliedAt = appliedAt,
						data = {},
						spellId = spellId,
						auraType = "BUFF",
					}
				end
			end
			
			table.wipe (cachedAuraTable)
		end
	end
	
	function castHistory.OnCombatStart (combatObject)
		--start
		castHistory.inCombat = true
		castHistory.currentCombat =  combatObject
		--built the table
		DF:GroupIterator (castHistory.BuildPlayerList, combatObject)
		--precasts
		DF:GroupIterator (castHistory.CacheTransfer, combatObject, spellCache, auraCache)
	end	
	
	function castHistory.OnCombatEnd()
		--finish
		castHistory.inCombat = false
		
		--close aura timers
		if (castHistory.currentCombat) then
			for GUID, playerAuraTable in pairs (castHistory.currentCombat.aura_timeline) do
				for spellId, auraTable in pairs (playerAuraTable) do
					if (auraTable.enabled) then
						auraTable.enabled = false
						tinsert (auraTable.data, {auraTable.appliedAt, GetTime()}) --when it was applied and when it went off
					end
				end
			end
		end
	end
	
	function castHistory.InstallTab()
		local tabName = "CastTimeline"
		local tabNameLoc = "Cast Timeline (New Plugin Pls Test)"
		
		local canShowTab = function (tabOBject, playerObject)
			local combat = Details:GetCombatFromBreakdownWindow()
			if (combat) then
				if (combat.spells_cast_timeline [playerObject.serial]) then
					tabOBject.frame:Show()
					return true
				end
			end
			return false
		end
		
		local fillTab = function (tab, playerObject, combat)

			local castData = combat.spells_cast_timeline [playerObject and playerObject.serial]
			if (castData and castData[1]) then
				local spellData = {}
				
				local firstAbilityUsedTime = castData[1][1] --gettime
				local combatTime = combat:GetCombatTime() --seconds elapsed
				local combatStartTime = combat:GetStartTime() --gettime
				local combatEndTime = combat:GetEndTime() --gettime

				--cast data
				for i = 1, #castData do
					local cast = castData [i]
					local atTime = cast [1]
					local spellId = cast [2]
					
					spellData [spellId] = spellData [spellId] or {}
					tinsert (spellData [spellId], {atTime - combatStartTime, 1})
				end
				
				local scrollData = {
					length = combatTime,
					defaultColor = {1, 1, 1, 1},
					useIconOnBlocks = true,
					lines = {},
				}
				
				for spellId, spellTimers in pairs (spellData) do
					local t = {
						text = GetSpellInfo (spellId),
						icon = select (3, GetSpellInfo (spellId)),
						spellId = spellId,
						timeline = spellTimers,
					}
					tinsert (scrollData.lines, t)
				end
				
				--aura data
				local auraData = combat.aura_timeline [playerObject and playerObject.serial]
				if (auraData) then
					local auraTimeData = {}
					for spellId, auraTable in pairs (auraData) do
						local auraTimers = {}
						
						for i = 1, #auraTable.data do
							local auraActivationData = auraTable.data [i]
							local applied = auraActivationData [1]
							local wentOff = auraActivationData [2]
							local duration = wentOff - applied

							tinsert (auraTimers, {applied - combatStartTime, 10, duration,  duration})
						end
						
						local t = {
							text = GetSpellInfo (spellId),
							icon = select (3, GetSpellInfo (spellId)),
							spellId = spellId,
							timeline = auraTimers,
						}
						
						tinsert (scrollData.lines, t)
					end
				end
				
				tab.frame.scroll:SetData (scrollData)
			end
		end
		
		local onClickTab = function ()
			tab.frame:Show()
		end
		
		local onCreatedTab = function (tab, frame)
			local scroll = DF:CreateTimeLineFrame (frame, "$parentTimeLine",
			{ --general options
				width = 865, height = 438,
			
				backdrop_color = {1, 1, 1, .1},
				backdrop_color_highlight = {1, 1, 1, .3},

				slider_backdrop_color = {.2, .2, .2, 0.3},
				slider_backdrop_border_color = {0.2, 0.2, 0.2, .3},
				
				on_enter = function (self)
					self:SetBackdropColor (unpack (self.backdrop_color_highlight))
				end,
				
				on_leave = function (self)
					if (self.dataIndex % 2 == 1) then
						self:SetBackdropColor (1, 1, 1, 0)
					else
						self:SetBackdropColor (1, 1, 1, .1)
					end
				end,
				
				block_on_enter = function (self)
					local info = self.info
					local spellName, _, spellIcon = GetSpellInfo (info.spellId)
					GameCooltip2:Preset (2)
					GameCooltip2:SetOwner (self)
					GameCooltip2:AddLine (spellName)
					GameCooltip2:AddIcon (spellIcon, 1, 1, 16, 16, .1, .9, .1, .9)
					GameCooltip2:AddLine ("Time:", info.time < -0.2 and "-" .. DF:IntegerToTimer (abs (info.time)) or DF:IntegerToTimer (abs (info.time)))
					if (info.duration) then
						GameCooltip2:AddLine ("Duration:", DF:IntegerToTimer (info.duration))
					end
					GameCooltip2:Show()
					
					self.icon:SetBlendMode ("ADD")
				end,
				
				block_on_leave = function (self)
					GameCooltip2:Hide()
					self.icon:SetBlendMode ("BLEND")
				end,
			},
			
			{ --timeline frame options
				draw_line_color = {1, 1, 1, 0.03},
			}
			)

			local scrollText = DF:CreateLabel (scroll.horizontalSlider, "scroll", 10, {.5, .5, .5, .5})
			scrollText:SetPoint ("right", scroll.horizontalSlider, "right", -2, 0)

			local zoomText = DF:CreateLabel (scroll.scaleSlider, "zoom", 10, {.5, .5, .5, .5})
			zoomText:SetPoint ("right", scroll.scaleSlider, "right", -2, 0)

			scroll:SetPoint ("topleft", frame, "topleft", 0, 0)
			frame.scroll = scroll
		end
		
		local iconSettings = {
			texture = [[Interface\CHATFRAME\CHATFRAMEBACKGROUND]],
			coords = {0.1, 0.9, 0.1, 0.9},
			width = 16,
			height = 16,
		}

		Details:CreatePlayerDetailsTab (tabName, tabNameLoc, canShowTab, fillTab, nil, onCreatedTab, iconSettings)
	end
	
	function castHistory:OnEvent (_, event, ...)
	
		if (event == "UNIT_AURA") then
			local unitId = ...
			if (unitId and UnitIsPlayer (unitId) and (UnitInParty (unitId) or UnitInRaid (unitId) or UnitIsUnit (unitId, "player"))) then
				local GUID = UnitGUID (unitId)
				if (GUID) then
					if (castHistory.inCombat) then
						local playerAuraTable = castHistory.currentCombat.aura_timeline [GUID]
						if (playerAuraTable) then
							local aurasFound = {}
							for i = 1, 40 do
								local name, texture, count, debuffType, duration, expirationTime, caster, canStealOrPurge, nameplateShowPersonal, spellId = UnitBuff (unitId, i)
								
								if (not name) then
									break
								end
								
								if (duration and duration > 0 and duration < 50) then
									local auraTable = playerAuraTable [spellId]
									if (not auraTable) then 
										playerAuraTable [spellId] = {
											enabled = true,
											appliedAt = GetTime(),
											data = {},
											spellId = spellId,
											auraType = "BUFF",
										}
										auraTable = playerAuraTable [spellId]
									else
										if (not auraTable.enabled) then
											auraTable.enabled = true
											auraTable.appliedAt = GetTime()
										end
									end
									
									aurasFound [spellId] = true
								end
							end
							
							for spellId, auraTable in pairs (playerAuraTable) do
								if (auraTable.enabled and not aurasFound [spellId]) then
									auraTable.enabled = false
									tinsert (auraTable.data, {auraTable.appliedAt, GetTime()}) --when it was applied and when it went off
								end
							end
						end
					else
						--details! isn't in combat
						--add the spell into the cache
						local cachedAuraTable = auraCache [GUID] or {}
						auraCache [GUID] = cachedAuraTable

						for i = 1, 40 do
							local name, texture, count, debuffType, duration, expirationTime, caster, canStealOrPurge, nameplateShowPersonal, spellId = UnitBuff (unitId, i)
							
							if (not name) then
								break
							end
							
							if (duration and duration > 0 and duration < 50) then
								cachedAuraTable [spellId] = expirationTime - duration
							end
						end
					end
				end
			end
		
		elseif (event == "UNIT_SPELLCAST_SUCCEEDED") then
			local unitId, _, spellId = ...
			if (unitId and spellId and UnitIsPlayer (unitId) and (UnitInParty (unitId) or UnitInRaid (unitId) or UnitIsUnit (unitId, "player"))) then
				local GUID = UnitGUID (unitId)
				if (GUID) then
					if (castHistory.inCombat) then
						local playerCastTable = castHistory.currentCombat.spells_cast_timeline [GUID]
						if (playerCastTable) then
							tinsert (playerCastTable, {GetTime(), spellId})
						end

					else
						local playerCastTable = spellCache [GUID]
						
						if (not playerCastTable) then
							spellCache [GUID] = {}
							playerCastTable = spellCache [GUID]
						else
							local cutOffTime = GetTime() - 5
							for i = #playerCastTable, 1, -1 do
								if (playerCastTable [i][1] < cutOffTime) then
									tremove (playerCastTable, i)
								end
							end
						end

						tinsert (playerCastTable, {GetTime(), spellId})
					end
				end
			end
		
		elseif (event == "ADDON_LOADED") then
			local AddonName = select (1, ...)
			if (AddonName == "Details_CastHistory") then
				
				--> every plugin must have a OnDetailsEvent function
				function castHistory:OnDetailsEvent (event, ...)
					return handle_details_event (event, ...)
				end
				
				--> Install: install -> if successful installed; saveddata -> a table saved inside details db, used to save small amount of data like configs
				local install, saveddata = Details:InstallPlugin ("RAID", "Cast History", "Interface\\Icons\\Ability_Warrior_BattleShout", castHistory, "DETAILS_PLUGIN_CAST_HISTORY", MINIMAL_DETAILS_VERSION_REQUIRED, "Details! Team", CASTHISTORY_VERSION)
				if (type (install) == "table" and install.error) then
					print (install.error)
				end
				
				--> registering details events we need
				Details:RegisterEvent (castHistory, "COMBAT_PLAYER_ENTER") --when details creates a new segment, not necessary the player entering in combat.
				Details:RegisterEvent (castHistory, "COMBAT_PLAYER_LEAVE") --when details finishs a segment, not necessary the player leaving the combat.
				Details:RegisterEvent (castHistory, "DETAILS_DATA_RESET") --details combat data has been wiped
				
				castHistory.Frame:RegisterEvent ("UNIT_SPELLCAST_SUCCEEDED")
				castHistory.Frame:RegisterEvent ("UNIT_AURA")
				
				castHistory.InstallTab()
			end
		end

	end

end
